@@ -28,20 +28,24 @@
 def createSolidMeshes( rp, runTime ):
     
     from Foam.finiteVolume import PtrList_fvMesh, fvMesh
-    solidRegions = PtrList_fvMesh( rp.solidRegionNames().size() )
+    solidRegions = PtrList_fvMesh( rp.solidRegionNames.size() )
     
     from Foam.OpenFOAM import word, fileName, IOobject
     for index in range( solidRegions.size() ):
         from Foam.OpenFOAM import ext_Info, nl
-        ext_Info()<< "Create solid mesh for region " << rp.solidRegionNames()[ index ] \
+        ext_Info()<< "Create solid mesh for region " << rp.solidRegionNames[ index ] \
             << " for time = " << runTime.timeName() << nl << nl
         
-        solidRegions.ext_set(index, fvMesh( IOobject ( word( rp.solidRegionNames()[ index ] ),
+        solidRegions.ext_set(index, fvMesh( IOobject ( word( rp.solidRegionNames[ index ] ),
                                                        fileName( runTime.timeName() ),
                                                        runTime,
                                                        IOobject.MUST_READ ) ) )
         pass
-
+    # Force calculation of geometric properties to prevent it being done
+    # later in e.g. some boundary evaluation
+    # (void)solidRegions[i].weights();
+    # (void)solidRegions[i].deltaCoeffs();
+    
     return solidRegions
 
 
@@ -56,6 +60,7 @@ def createSolidField( solidRegions, runTime ):
     Ks = PtrList_volScalarField( solidRegions.size() )
     Ts = PtrList_volScalarField( solidRegions.size() )
     
+        
     from Foam.OpenFOAM import ext_Info, nl
     
     #Populate solid field pointer lists    
@@ -65,6 +70,7 @@ def createSolidField( solidRegions, runTime ):
             << solidRegions[ index ].name() << nl << nl
 
         ext_Info()<< "    Adding to rhos\n" << nl
+        
         rhos.ext_set( index, volScalarField( IOobject ( word( "rho" ), 
                                                         fileName( runTime.timeName() ), 
                                                         solidRegions[ index ], 
@@ -89,9 +95,8 @@ def createSolidField( solidRegions, runTime ):
                                                      IOobject.MUST_READ,
                                                      IOobject.AUTO_WRITE ),
                                            solidRegions[ index ] ) )
-        
+                
         ext_Info()<< "    Adding to Ts\n" << nl
-       
         Ts.ext_set( index, volScalarField( IOobject( word( "T" ),
                                                      fileName( runTime.timeName() ),
                                                      solidRegions[ index ],
@@ -103,72 +108,23 @@ def createSolidField( solidRegions, runTime ):
 
 
 #-----------------------------------------------------------------------------------------------------
-def readSolidMultiRegionPIMPLEControls( mesh ):
-    from Foam.OpenFOAM import word
-    pimple = mesh.solutionDict().subDict( word( "PIMPLE" ) )
-    nNonOrthCorr = pimple.lookupOrDefault( word( "nNonOrthogonalCorrectors" ), 0 )
+def readSolidMultiRegionPISOControls( mesh ):
+    from Foam.OpenFOAM import word, readInt
+    piso = mesh.solutionDict().subDict( word( "PISO" ) )
     
-    return pimple, nNonOrthCorr
+    nNonOrthCorr = 0
+    if piso.found( word( "nNonOrthogonalCorrectors" ) ):
+       nNonOrthCorr = readInt( piso.lookup( word( "nNonOrthogonalCorrectors" ) ) )
+
+    return piso, nNonOrthCorr
 
 
 #-------------------------------------------------------------------------------------------------------
-def readSolidTimeControls( runTime ):
-    from Foam.OpenFOAM import word 
-    maxDi = runTime.controlDict().lookupOrDefault( word( "maxDi" ) , 10.0 )
-    
-    return maxDi
-
-
-#-------------------------------------------------------------------------------------------------------
-def solidRegionDiffNo( mesh, runTime, Cprho, K ):
-    DiNum = 0.0
-    meanDiNum = 0.0
-
-    #- Can have fluid domains with 0 cells so do not test.
-    if mesh.nInternalFaces():
-       from Foam import fvc
-       KrhoCpbyDelta = mesh.deltaCoeffs() * fvc.interpolate( K ) / fvc.interpolate(Cprho);
-       DiNum = KrhoCpbyDelta.internalField().max() * runTime.deltaT().value()
-       meanDiNum = KrhoCpbyDelta.average().value() * runTime.deltaT().value()
-       pass
-    
-    from Foam.OpenFOAM import ext_Info, nl
-    ext_Info() << "Region: " << mesh.name() << " Diffusion Number mean: " << meanDiNum << " max: " << DiNum << nl
-
-    return DiNum
-
-
-#-------------------------------------------------------------------------------------------------------
-def solidRegionDiffusionNo( solidRegions, runTime, rhosCps, Ks ):
-    from Foam.OpenFOAM import GREAT
-    DiNum = -GREAT
-    
-    for regionI in range( solidRegions.size() ):
-        DiNum = max( solidRegionDiffNo( solidRegions[ regionI ], runTime, rhosCps[ regionI ], Ks[ regionI ] ), DiNum )
+def solveSolid( i, rhosCps,  Ks, Ts, nNonOrthCorr ):
+    for nonOrth in range( nNonOrthCorr +1 ):
+        from Foam.finiteVolume import solve
+        from Foam import fvm
+        solve( fvm.ddt( rhosCps[ i ], Ts[ i ]) - fvm.laplacian( Ks[ i ], Ts[ i ] ) )
         pass
-    
-    return DiNum
-
-#-------------------------------------------------------------------------------------------------------
-def setRegionSolidFields( i, solidRegions, rhos, cps, Ks, Ts ):
-    mesh = solidRegions[ i ]
-    rho = rhos[ i ]
-    cp = cps[ i ]
-    K = Ks[ i ]
-    T = Ts[ i ]
-    
-    return mesh, rho, cp, K, T
-
-
-#-------------------------------------------------------------------------------------------------------
-def solveSolid( mesh, rho, cp, K, T, nNonOrthCorr ):
-    for index in range( nNonOrthCorr + 1 ):
-       from Foam import fvm
-       TEqn = fvm.ddt( rho * cp, T ) - fvm.laplacian( K, T )
-       TEqn.relax()
-       TEqn.solve()
-       pass
-    from Foam.OpenFOAM import ext_Info, nl   
-    ext_Info()<< "Min/max T:" << T.ext_min() << ' ' << T.ext_max() << nl
-    
+      
     pass
